/*     mmfir.c: MiniMax FIR filter design per McClellan, Parks & Rabiner
 *        Copyright (c) 2014/15 Rob Sykes <robs@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Design goals for this implementation:
 *   Speed:            Medium         Data size:    Medium
 *   Code size:        Medium         Robustness:   Very high
 *   Code complexity:  Medium         Portability:  C89 + inline, C++
 *
 * Other licences/implementations may be available from the author.
 */



#if !defined(GNU_OCTAVE)
  #include "mmfir.h"
#else /* Octave prefers everything in one file (named per the Octave fn.). */
/* ----------------------------- mmfir.h begins ----------------------------- */
#if !defined mmfir_included
#define mmfir_included



typedef enum {
  mmfirSymmetric,          /* Type I/II (symmetric impulse-response). */
  mmfirAntiSymmetric,      /* Type III/IV (anti-symmetric impulse-response). */
  mmfirDifferentiator      /* Ditto, but with passband error-weighting *= 2/f */
} mmfirFilterClass;



typedef struct {
  double freqL, freqR;     /* Frequency at left & right band-edges. In [0,1] */
  double ampL, ampR;       /* Desired amplitude of response at given edges. */
  double weightL, weightR; /* Error weighting (multiplier) at given edges. */

  /* Internal use only: */
  int ends, weightF, endP, hadRemoval;
  double eAv, ePk;
  double portion0, portion;
} mmfirBandSpec;



typedef enum {
  mmfirSuccess,

  mmfirWarning,            /* Section marker */
  mmfirGaveUp = mmfirWarning,
  mmfirGaveUp1,            /* During first pass. */

/* No filter will be returned: */
  mmfirMissedTarget,
  mmfirError,              /* Section marker */
  mmfirTooFewPeaks = mmfirError,
  mmfirTooManyPeaks,
  mmfirNumericalError,

  mmfirInvocationError,    /* Section marker */
  mmfirOutOfMemory = mmfirInvocationError,
  mmfirInvalidNumBands,
  mmfirInvalidOrder,
  mmfirInvalidFrequency,
  mmfirInvalidWeight,
  mmfirInvalidAmplitude,
  mmfirInvalidDcAmplitude,
  mmfirInvalidNyquistAmplitude,

/* Internal use only: */
  mmfirRuntimeError,
  mmfirOngoing
} mmfirResult;



typedef struct {
  /* Useful stuff: */
  mmfirResult result;      /* As defined above. */
  char const * text;       /* Textual version of result. */
  double minimax;          /* Final approximated minimax weighted error. */

  /* Academic interest: */
  int FEs;                 /* # of Lagrange interp. fn. evaluations made. */
  int iterations;          /* # of Remez iterations made. */

  double controls[3];      /* Allows client to know if these were clamped. */
} mmfirReport;



#if defined __cplusplus
extern "C"
#endif

mmfirReport mmfir(
    double            h[], /* In which to store the N filter coeffiecients. */
    mmfirFilterClass  filterClass,
    int               N, /* = filter order + 1 */
    int               numBands,
    mmfirBandSpec     bandSpecs[/* numBands */], /* N.B. destroyed. */
    double            accuracy,
    double            persistence,
    double            robustness,
    double            target,
    int               flags);



#endif
/* ------------------------------ mmfir.h ends ------------------------------ */
#endif



#if defined QUAD_PREC
  #include <quadmath.h>
#endif

#if defined(__cplusplus)
  #include <climits>
  #include <cmath>
  #include <cstdlib>
  #include <cstdio>
  #include <cstring>
#else
  #include <limits.h>
  #include <math.h>
  #include <stdlib.h>
  #include <stdio.h>
  #include <string.h>
#endif

#if INT_MAX == 2147483647
  typedef int rand_t;
#elif LONG_MAX == 2147483647L
  typedef long rand_t;
#else
  #error no suitable int32 type
#endif

#if defined _MSC_VER
  #define inline __inline
#endif



/* Real-number precision, quad, double or long double (the default). */

#if defined QUAD_PREC /* Very slow without native support. */
  #define real __float128
  #define cosr cosq
  #define sinr sinq
  #define PIr M_PIq
#elif defined DOUBLE_PREC
  #define real double
  #define cosr cos
  #define sinr sin
  #define PIr PI
#else
  #define real long double
  #define cosr cosl
  #define sinr sinl
  #define PIr 3.1415926535897932384626433832795029L
#endif



#define bandL           (&bandSpecs[0])
#define bandR           (&bandSpecs[numBands-1])
#define clamp(x,lo,hi)  (((x)>(hi))?(hi):(((x)<(lo))?(lo):(x)))
#define debugLevel      (flags & 3)
#define D(f)            (D0(f) / typeMod(f,T)) /* Desired response fn */
#define dir(x)          ((x)<0? -1: (x)>0? 1 : 0)
#define epsilon         1e-11 /* In some circumstances, treat < epsilon as 0 */
#define E(x)            peaks[x].e
#define e(x)            space[x].e
#define F(x)            peaks[x].f
#define f(x)            space[x].f
#define maxExtras       (numBands+2)
#define maxPeaks        (R+1+maxExtras)
#define PI              3.1415926535897932384626433832795029
#define rand(r,x)       (ranqd1(r) * ((x) / (65536. * 32768.))) /* in [-x,x) */
#define ranqd1(r)       ((r) = 1664525 * (r) + 1013904223) /* int32_t r */
#define returnError(x)  do {report.result=x; goto END;} while(0)
#define round(x)        (int)floor((x)+.5)
#define setF(F)         for (f=(F); f>b->freqR; ++b)
#define userEpsilon     1e-6
#define W(f)            (W0(f)/(b->weightF?f:1)*typeMod(f,T)) /* Weight fn */



/* Less problematic (side-effects, compiler bugs) than equivalent macro: */
static inline double maxD(double a, double b) {return a>=b? a:b;}
static inline int    maxI(int    a, int    b) {return a>=b? a:b;}



/* These 2 macros could be replaced with function pointers to allow
 * client-supplied, arbitrary weighting or desired amplitude response: */

#if 1
  /* '...R' values here contain pre-calculated gradient: */
  #define W0(f) (b->weightL + b->weightR*(f-b->freqL)) /* Weighting function */
  #define D0(f) (b->ampL    + b->ampR   *(f-b->freqL)) /* Desired response fn */
#else /* Along the lines of: */
  extern double (* W0)(double f);
  extern double (* D0)(double f);
#endif



static char const * const errorText[]={
  "out-of-memory",
  "number of bands must be at least 1",
  "filter-order must be at least 1",
  "band-edge frequencies must increase in [0,1]",
  "weighting function out-of-range",
  "amplitude function out-of-range",
  "type III/IV DC amplitude response must be 0",
  "type II/III Nyquist amplitude response must be 0",
};



/* Type definitions: */

typedef enum {TypeI, TypeII, TypeIII, TypeIV} FirType;

typedef struct {real x, beta, gamma;} LagrangeCoef;

typedef struct {
  double f;    /* Frequency in [0,1] of this point */
  double e;    /* Error at this point */
} Point;



/* Determine coefs of a 2nd-order polynomial from 3 of its points: */

static void poly2(
    double x0, double x1, double x2,
    double y0, double y1, double y2,
    double * a, double * b, double * c)
{
  double a0 = y0/((x0-x1)*(x0-x2));
  double a1 = y1/((x1-x2)*(x1-x0));
  double a2 = y2/((x2-x0)*(x2-x1));
  *a = a0+a1+a2;
  *b = -(a0*(x1+x2) + a1*(x0+x2) + a2*(x0+x1));
  *c = a0*x1*x2 + x0*a1*x2 + x0*x1*a2;
}



/* Evaluate a 2nd-order polynomial at given point: */

static double poly2Val(double a, double b, double c, double x)
{
  return (a*x + b)*x + c;
}



/* For filter-response analysis, use a discrete frequency space that is
 * 'warped' around transition-bands.  The warping function is derived from the
 * observation by Shpak/Antoniou (1990) that an 'initial guess' for the peak
 * frequencies based on adjacent-peak spacing proportional to s(n) =
 * exp(c1+c2*n), improves upon one where the adjacent-peak spacing is constant.
 *
 * E.g., for a band-pass filter with band-edges 0, s1, p1, p2, s2, 1, the
 * analysis points within warped space compared to those in a linear space
 * might look like the following (second stop-band not shown):
 *          0                       s1    p1                                  p2
 * Warped:  o      o     o    o  o oo     oo o  o    o      o      o    o  o oo
 * Linear:  o o o o o o o o o o o o o     o o o o o o o o o o o o o o o o o o o
 *
 * Warp function derivation is as follows: approx. peak position continuous
 * function, warp(x) =
 *   x>=1: f(x)-f(0)
 *   x<1:  2nd-order approximation (since exp. approx. breaks down here)
 * where f(x) = Indef. integral of s(x) = exp(c1+c2*x)/c2
 *
 * Constants c1, c2 are determined empirically.  Dependent constants c3=-f(0),
 * ca,cb,cc are calculated as follows: c3 = -exp(c1)/c2;
 *   poly2(0,1,2, 0,warp(1),warp(2), &ca,&cb,&cc);
 */

static double const c1 = +.1235;
static double const c2 = -.8;

static double const c3 = 1.41431;
static double const ca = .214437;
static double const cb = .006742;
static double const cc = 0;



static double warp(double x) /* Calculate warp in arbitrary units: */
{
  double t = c1 + c2*x;
  return x<0? 0 : x<1? poly2Val(ca,cb,cc,x) : x-(t>-38? exp(t)/c2 : 0)-c3;
}



static double iwarp(double y) /* Inverse of warp(). */
{
  double t, x1=y+c3, x;

  if (y<0)
    return 0;

  if (y<warp(1))
    return (-cb + sqrt(cb*cb-4*ca*(cc-y)))/(2*ca);

  /* Newton iteration: */
  do x=x1, t=c1+c2*x, x1=x-(warp(x)-y)/(1-(t>-38? exp(t) : 0));
  while (fabs(x1-x)>1e-5);
  return x1;
}



/* Generate a set of warped frequency-space points.
 * N.B. rand helps to avoid Lagrange interpolation instability: */

static int warpSpace(Point * p, double m, double f1, double f2,
    int density, int step, rand_t * random)
{
  int i, n = (int)ceil(m*density);
  double scale, df = f2 - f1;

  p->f = f1, p+=step; /* End points must be exact (not calculated). */
  if (n > 0) {
    scale = df/warp(1.*n/density);
    for (i = 1; i < n; ++i)
      p->f = f1+scale*warp((i+rand(*random, .001))/density), p+=step;
    p->f = f2; /* End points must be exact (not calculated). */
  }
  return 1+n;
}



static int warpSpaceR( /* Right of transition-band. */
    Point * p, double m, double f1, double f2, int density, rand_t * random)
{
  return warpSpace(p, m, f1, f2, density, +1, random);
}



static int warpSpaceL ( /* Left of transition-band. */
    Point * p, double m, double f1, double f2, int density, rand_t * random)
{
  return warpSpace(p+(int)ceil(m*density), m, f2, f1, density, -1, random);
}



static int warpSpaceRL( /* Between transition-bands. */
    Point * p, double m, double f1, double f2, int density, rand_t * random)
{
  double df1 = (f2 - f1) * .5, m1 = m * .5;
  int n = warpSpaceR(p, m1, f1, f1+df1, density, random);
  warpSpaceL(p + n-1, m1, f1+df1, f2, density, random);
  return n*2-1;
}



/* Apportion space to each band, taking account of warping: */

static void apportionSpace(mmfirBandSpec * b, int numBands, int n)
{
  int i, changed;
  double sum, t, av;

  do { /* Use an iterative method. */
    for (av=changed=0, i=0; i<numBands; ++i)
      av += warp(b[i].portion*n/b[i].ends)/(b[i].portion0/b[i].ends);
    av/=numBands;

    for (sum=i=0; i<numBands; ++i) {
      t=iwarp(av*b[i].portion0/b[i].ends)/n*b[i].ends;
      changed += fabs(t-b[i].portion) > 1e-6;
      sum+= b[i].portion=t;
    }
    for (i=0; i<numBands; b[i++].portion/=sum); /* Normalise. */
  } while (changed);
}



static inline real /* Modifier at given f in [0,1], for filter-type T */
typeMod(real f, FirType T)
{
  switch (T) {
    case TypeI  : default: return 1;
    case TypeII : return cosr(f *(PIr*.5)); /* Has zero at f=1. */
    case TypeIII: return sinr(f * PIr);     /* Has zero at f=0, f=1. */
    case TypeIV : return sinr(f *(PIr*.5)); /* Has zero at f=0. */
  }
}



/* Calculate the estimated amplitude response at a single, given frequency
 * using precalculated Lagrange interpolation coefficients. */

static real A(LagrangeCoef * coefs, int R, double f, int * FEs)
{
  int i = 0;
  real n = 0, d = 0, x = cosr(PIr*f), t;
  ++*FEs;
  for (; i < R; d += t = coefs[i].beta / t, n += coefs[i].gamma * t, ++i)
    if (fabs((double)(t = x - coefs[i].x)) < epsilon)
      return coefs[i].gamma;
  return n/d;
}



mmfirReport mmfir(
    double            h[],
    mmfirFilterClass  filterClass,
    int               N0, /* = filter order + 1 */
    int               numBands,
    mmfirBandSpec     bandSpecs[/* numBands */], /* N.B. destroyed. */
    double            accuracy,
    double            persistence,
    double            robustness,
    double            target,
    int               flags)
{
  int      N = maxI(N0,2);
  FirType  T = (FirType)((filterClass!=mmfirSymmetric)*2 + !(N&1));
  int      R = N/2 + !T; /* R = # of cosine functions; num peaks = R+1 */

  mmfirReport report = {mmfirSuccess, 0, 0, 0, 0, {0,0,0}};

  /* For forward-compatibility of the use of the control
   * variables, (almost) silently ignore out-of-range values: */
  int      density2 = round(5+5*(
             report.controls[0]=clamp(accuracy,0,7)     ));
  int      maxIterations = round(40*pow(1.5,
             report.controls[1]=clamp(persistence,-5,5) ));
  int      density = 3+(numBands>2)+round(
             report.controls[2]=clamp(robustness,0,3)   );

  int      doneInit=0, doLG=1; /* For the lower/higher density passes. */
  int      spaceLength, i, j, _1; /* _1 holds +/- 1 */
  int      prevNumPeaks=0;
  double   f, maxA=0;
  double   delta_1 = 1e-30; /* 1/delta */
  double   totalWidth; /* Of given bands; in which peaks will be distributed. */
  mmfirBandSpec * b;

  /* Allocate the working arrays: */
  LagrangeCoef * coefs = (LagrangeCoef *)calloc((size_t)(R+1), sizeof(*coefs));
  Point       * peaks = (Point *)calloc((size_t)(maxPeaks), sizeof(*peaks));
  double  * prevPeaks =(double *)calloc((size_t)(maxPeaks), sizeof(*prevPeaks));
  Point      * space0 = 0, * space = 0; /* Need to determine its size first. */

  /* Initial checks: */
  if (numBands<=0)
    returnError(mmfirInvalidNumBands);
  if (N!=N0)
    returnError(mmfirInvalidOrder);
  if (bandL->freqL<0 || bandR->freqR>1)
    returnError(mmfirInvalidFrequency);

  /* Further checks, and pre-calc. linear interp. gradients, etc.: */
  for (totalWidth=0, b=bandL; b<=bandR; ++b) {
    double df = b->freqR - b->freqL;

    if (fabs(b->ampL) > 1e+10 || fabs(b->ampR) > 1e+10)
      returnError(mmfirInvalidAmplitude);

    b->weightF = (fabs(b->ampL) > userEpsilon || fabs(b->ampR) > userEpsilon) &&
      filterClass==mmfirDifferentiator;

    if (b->weightL < 1e-10 || b->weightL > 1e+10 ||
        b->weightR < 1e-10 || b->weightR > 1e+10)
      returnError(mmfirInvalidWeight);
    else if (b->weightF)
      b->weightL *= 2, b->weightR *= 2; /* For 2/f weighting. */

    if ((df <= userEpsilon) || (b<bandR && b->freqR>(b+1)->freqL))
      returnError(mmfirInvalidFrequency);

    b->ampR    = (b->ampR    - b->ampL   ) / df;
    b->weightR = (b->weightR - b->weightL) / df;
    totalWidth += df;

    maxA=maxD(maxA, maxD(fabs(b->ampL),fabs(b->ampR)));
  }
  if (maxA<1e-20)
    returnError(mmfirInvalidAmplitude);

  /* Avoid dividing by typeMod(0,T)=0: */
  if ((T == TypeIII || T == TypeIV) && (b=bandL)->freqL < (f=epsilon)) {
    if (b->ampL > userEpsilon)
      returnError(mmfirInvalidDcAmplitude);

    b->ampL += b->ampR*(f-b->freqL);
    b->weightL += b->weightR*(f-b->freqL);
    totalWidth -= f-b->freqL, b->freqL = f;
  }

  /* Avoid dividing by typeMod(1,T)=0: */
  if ((T==TypeII || T==TypeIII) && (b=bandR)->freqR > (f=1-epsilon)) {
    if (D0(b->freqR) > userEpsilon)
      returnError(mmfirInvalidNyquistAmplitude);

    totalWidth -= b->freqR-f, b->freqR = f;
  }

  /* Separate any contiguous bands: */
  for (b=bandL; b<bandR; ++b)
    if (b->freqR > (f = b[1].freqL-epsilon))
      totalWidth -= b->freqR-f, b->freqR = f;

  /* Usually, start with a lower density, then switch to the final value: */
  for (density2=maxI(density,density2);
      report.result==mmfirSuccess && density<=density2;
      doLG=0, density+=maxI(density2-density,1)) {

    /* Map current density to a convergence minimality threshold: */
    double minimalityThreshold = density<density2? density*.35-.45 :
      (density+35)*.05; /* Though in this case, stability usually prevails. */
    if (debugLevel>2)
      fprintf(stderr, "minimalityThreshold: %g\n", minimalityThreshold);

    { /* Allocate & populate the analysis frequency space: */
      rand_t random = 0;

      for (b=bandL; b<=bandR; ++b) {
        double df = b->freqR - b->freqL;
        b->portion = b->portion0 = df/totalWidth;
        b->ends=2-(b->freqL <= epsilon || b->freqR >= 1-epsilon);
      }
      apportionSpace(bandL, numBands, R+1);

      /* Determine # of points in analysis space: */
      for (spaceLength=0, b=bandL; b<=bandR; ++b) {
        double m = (R+1)*b->portion;
        if (b->freqL <= epsilon || b->freqR >= 1-epsilon)
          spaceLength += 1 + (int)ceil(m*density);
        else
          spaceLength += 2 * (1 + (int)ceil(m*.5*density)) - 1;
      }

      /* Allocate space and calculate frequencies: */
      free(space0);
      if ((space0 = (Point *)calloc((size_t)(spaceLength+2), sizeof(*space)))) {
        space = space0+1; /* Simplifies peak detection at edges of space. */
        for (j=0, b = bandL; b<=bandR; ++b) {
          double m = (R+1)*b->portion;
          if (b->freqL <= epsilon)
            j+= warpSpaceL(space+j, m, b->freqL, b->freqR, density, &random);
          else if (b->freqR >= 1-epsilon)
            j+= warpSpaceR(space+j, m, b->freqL, b->freqR, density, &random);
          else
            j+= warpSpaceRL(space+j, m, b->freqL, b->freqR, density, &random);
        }
      }
    }

    if (!space0 || !coefs || !peaks || !prevPeaks)
      returnError(mmfirOutOfMemory);

    /* Initial 'guess' distributes peaks evenly through warped space.  The
     * offset of half a step at each end facilitates longer filters: */
    if (!doneInit++) for (i=0; i<=R; ++i)
      peaks[i].f=space[round((i+.5)/(R+1)*(spaceLength-1))].f;

    report.result=mmfirOngoing;
    while (1) { /* Perform the Remez exchange (until break, below): */
      double max, minimality=0, stability=0;
      int numPeaks, ok, extras=0;
      Point * p;

      if (doLG++) { /* Calculate Lagrange coefficients for response estimate: */
        real denom = 0, numer = 0, t1, t, delta;

        for (i=0; i<=R; ++i)
          coefs[i].x = cosr(PIr * peaks[i].f);

        for (_1 = -1, b=bandL, i=0; i <= R; ++i) {
          for (t=1, j=0; j<=R; t1=t, t*=2*(coefs[i].x-coefs[j].x) +(i==j), ++j);
          if (!t1 || !t)
            returnError(mmfirNumericalError);
          coefs[i].beta = 1/t1; /* Value at i==R will not be used.*/
          setF(peaks[i].f);
          numer += D(f)/t, denom += (_1 = -_1)/(t*W(f));
        }
        if (!numer || !denom)
          returnError(mmfirNumericalError);
        delta_1 = 1/(double)(delta = numer/denom);

        for (_1 = -1, b=bandL, i=0; i < R; ++i) {
          setF(peaks[i].f);
          coefs[i].gamma = D(f)-(_1 = -_1)*delta/W(f);
        }
      }

      /* Stop Remez here if converged or no iterations remain: */
      if (report.result == mmfirSuccess)
        break;
      else if (report.iterations == maxIterations) {
        report.result = density<density2? mmfirGaveUp1 : mmfirGaveUp;
        break;
      }
      else ++report.iterations;

      /* Evaluate the normalised (i.e. converges to [-1,1]) error function: */
      for (i=0, b=bandL, p=space; i<spaceLength; ++i, ++p) {
        setF(p->f);
        p->e = delta_1 * (double)(W(f) * (D(f) - A(coefs, R, f, &report.FEs)));
      }

      /* Find and store all local peaks in error magnitude: */
      b=bandL, numPeaks=0;
      for (i=0; i<spaceLength; ++i) if (dir(e(i+1)-e(i))!=dir(e(i)-e(i-1))) {
        double e, A, B, C, fp;

        /* Approx. continuous-space peak (by interpolation in discrete-space):*/
        setF(f(i));
        j = i - (f(i) == b->freqR) + (f(i) == b->freqL); /* Band-edge back-off*/
        poly2(f(j-1), f(j), f(j+1), e(j-1), e(j), e(j+1), &A, &B, &C);
        fp = B/A*-.5; /* Freq. at which 1st derivative of poly is 0. */
        ok = j==i || (dir(e(j+1)-e(j))*dir(e(j)-e(j-1)) > 0 &&
            fp >= f(i-(j<=i)) && fp <= f(i+(j>=i))); /* Band-edge needs care. */
        f = ok? fp : f, e = ok? poly2Val(A,B,C,f) : e(i);

        /* Store the peak:                             But avoid twin-    \_  */
        if (!numPeaks || f>peaks[numPeaks-1].f) {   /* peaks in this case:  \ */
          if (numPeaks == maxPeaks) /* Likely due to numerical error.*/
            returnError(mmfirTooManyPeaks);
          peaks[numPeaks].f = f, peaks[numPeaks++].e = e;
        }
      }

      if (debugLevel>1) { /* Dump arrays to files for debugging: */
        char name[9], dummy = (char)sprintf(name, "sp%i", report.iterations);
        FILE * f = fopen(name,"w") + 0*dummy;
        for (i=0; i<spaceLength;++i) fprintf(f, "%.16f % .9g\n", f(i), e(i));
        fclose(f);
        name[0]='p', f = fopen(name,"w");
        for (i=0; i<numPeaks;++i) fprintf(f, "%.16f % .9g\n", F(i), E(i));
        fclose(f);
      }

      /* Check that there are at least R+1 peaks: */
      if ((extras = numPeaks - (R+1)) < 0)
        returnError(mmfirTooFewPeaks); /* Perhaps due to numerical error. */

      /* Before discarding any peaks, check freq. stability of the entire set:*/
      for (max=1e-9, i=0; i<numPeaks-1; prevPeaks[i]=F(i), ++i)
        max=maxD(max,fabs(1-(prevPeaks[i+1]-prevPeaks[i])/(F(i+1)-F(i))));
      prevPeaks[i]=F(i); /* Since for-loop stops one short. */
      stability = numPeaks == prevNumPeaks? -log10(max) : -9.99;
      prevNumPeaks = numPeaks;

      /* If >R+1 peaks, reduce to R+1 by discarding lesser peaks/pairs: */
      while (numPeaks>R+1) {
        int n=1, try2, d_na, d; /* d = index of peak to discard. */

        /* Find a lesser peak, either overall or at one end: */
        try2=numPeaks-(R+1)>1, i=try2? 1 : numPeaks-1;
        for (d_na=d=0; !d_na && i<numPeaks; ++i) {
          d_na = E(i)*E(i-1)>0? i : d_na; /* Check alternating. */
          d = fabs(E(i)) < fabs(E(d))? i : d; /* Check magnitude. */
        }
        d=d_na? d_na: d; /* Prefer to discard non-alternating. */
        if (try2 && d && d != numPeaks-1) /* Discard pair only if not at end.*/
          n=2, d-=fabs(E(d-1))<fabs(E(d+1)); /* Choose lesser adjacent. */
        memmove(peaks+d, peaks+d+n, (size_t)((numPeaks-=n)-d)*sizeof(*peaks));
        if (debugLevel>2) fprintf(stderr, "x%i %i:%i\n", d_na, d, n);
      }

      /* Determine the minimality of the maximum error: */
      for (max=1+1e-9, i=0; i<=R; max=maxD(max, fabs(E(i))), ++i);
      minimality = -log10(max-1);

      /* Check for convergence: */
      if ((minimality>minimalityThreshold && (density<density2 || stability>2))
          || stability>2.7)
        report.result = mmfirSuccess;

      if (debugLevel) fprintf(stderr, "%2i %2i %2i % .2f % .2f\n",
        report.iterations, density, extras, maxD(minimality,-9.99), stability);
    }
  }

  if (target && 1/fabs(delta_1) > target)
    returnError(mmfirMissedTarget);

  { /* Generate filter coefficients: */
    double * a = (double *)space; /* Reuse space as it's no longer in use. */
    double _1 = "\1\377"[T>TypeII], phi = (T<TypeIII)*.5, s;

    /* Sample the final estimated response; modify for filter type: */
    for (i=0; i<=N/2; ++i)
      f=2.*i/N, a[i] = (double)(A(coefs, R, f, &report.FEs) * typeMod(f, T));

    /* --> time-domain using symmetry-aware IDFT (could also use IFFT): */
    for (a[N/2] /= 1+T/3, i=0; i <(N+1)/2; h[i] = _1*(h[N-1-i] = 2*s/N), ++i)
      for (s=*a*.5, j=1; j <= N/2; s += sin(PI*((N-1-2.*i)/N*j+phi))*a[j], ++j);
  }

END: /* Free working memory: */
  free(space0);
  free(prevPeaks);
  free(peaks);
  free(coefs);

  /* Complete and return the report: */
  report.text = report.result >= mmfirInvocationError?
    errorText[report.result-mmfirInvocationError] :
    report.result >= mmfirError?  "numerical-error" :
    report.result >= mmfirWarning?  "not-converged" : "success";
  report.minimax = 1/fabs(delta_1);
  return report;
}



#if defined(GNU_OCTAVE)
/* -------------------- Interface for GNU Octave; in C++ -------------------- */

#undef bandL
#undef bandR
#undef clamp
#undef cosr
#undef D
#undef D0
#undef debugLevel
#undef dir
#undef e
#undef E
#undef epsilon
#undef f
#undef F
#undef maxPeaks
#undef PI
#undef PIr
#undef rand
#undef ranqd1
#undef real
#undef returnError
#undef round
#undef setF
#undef sinr
#undef userEpsilon
#undef W
#undef W0



#include <octave/oct.h>
#include <octave/Cell.h>

#define AL(x) (int)(sizeof(x)/sizeof(x[0])) /* Array-Length */
#define returnError(...) do { error(__VA_ARGS__);\
  return octave_value_list(); } while (0)



DEFUN_DLD (firpm, args, , "\
-*- texinfo -*-\n\
@deftypefn  {Loadable Function} {@var{b} =} firpm (@var{n}, @var{f}, @var{a})\n\
@deftypefnx {Loadable Function} {@var{b} =} firpm (@var{n}, @var{f}, @var{a}, @var{w})\n\
@deftypefnx {Loadable Function} {@var{b} =} firpm (@var{n}, @var{f}, @var{a}, @var{w}, @var{properties})\n\
@deftypefnx {Loadable Function} {@var{b} =} firpm (@var{n}, @var{f}, @var{a}, @var{properties})\n\
@deftypefnx {Loadable Function} {[@var{b}, @var{minimax}] =} firpm (@dots{})\n\
@deftypefnx {Loadable Function} {[@var{b}, @var{minimax}, @var{result}] =} firpm (@dots{})\n\
\n\
Return as column-vector @var{b}, the coefficients of a linear-phase FIR\n\
filter designed according to given specifications and the `minimax'\n\
criterion.  The method (per McClellan et al.@footnote{ J. H. McClellan, T.\n\
W.  Parks and L. R. Rabiner, `A Computer Program for Designing Optimum FIR\n\
Linear Phase Digital Filters', IEEE Trans. Audio Electroacoust., vol. AU-21,\n\
1973, pp. 506-525.}) uses successive approximation to minimise the maximum\n\
weighted error between the desired and actual frequency response of the\n\
filter.  Filters thus designed are variably described as being `minimax',\n\
`equiripple', or `optimal (in the Chebyshev sense)'.\n\
\n\
@heading Arguments\n\
\n\
@table @var\n\
\n\
@item n\n\
A positive integer giving the filter order.\n\
\n\
@item f\n\
A vector of real-numbers, increasing in the range [0,1], giving the\n\
frequencies of the left and right edges of each band for which a specific\n\
amplitude response is desired: [l1 r1 l2 r2 @dots{}].  1 represents the\n\
Nyquist-frequency.  Transition-bands are defined implicitly as the regions\n\
between or outside the given bands.\n\
\n\
@item a\n\
A vector of real-numbers giving the desired amplitude response.  An amplitude\n\
value is given either for each band edge: [a(l1) a(r1) a(l2) a(r2) @dots{}], or\n\
for each band: [a1 a2 @dots{}].  In the former case, in-band amplitude is\n\
determined by linear interpolation between the given band-edge values.  1\n\
represents unity-gain, 0 represents infinite attenuation, and @minus{}1\n\
represents a phase change of pi radians.\n\
\n\
@item w\n\
An optional vector of positive real-numbers giving error-weightings to be\n\
applied at each given band-edge [w(l1) w(r1) w(l2) w(r2) @dots{}], or for\n\
each band [w1 w2 @dots{}].  In the former case, in-band weighting is\n\
determined by linear interpolation between the given band-edge values.  A\n\
higher relative error weighting yields a lower relative error.\n\
\n\
@item properties\n\
A value for a property is given by a pair of arguments: the first, a string,\n\
gives the property name; the second gives its value.  Available properties\n\
are as follows:\n\
\n\
@table @var\n\
@item class\n\
Denotes the filter-class: `symmetric' (the default) for type I or II filters,\n\
`antisymmetric' for standard type III or IV filters, or `differentiator' for\n\
type III or IV filters with error-weighting (further to @var{w}) of 2/f\n\
applied in the pass-band(s).  The string may be abbreviated.\n\
\n\
@ignore\n\
@c  This, firpmord, and perhaps a min-order search\n\
@c  function are for future consideration.\n\
@item target\n\
A positive real number giving a target minimax error; see Diagnostics, below.\n\
@end ignore\n\
\n\
@item accuracy\n\
Controls how close the computed filter will be to the ideal minimax solution.\n\
\n\
@item persistence\n\
Controls the number of computation iterations over which the required accuracy\n\
will be sought.\n\
\n\
@item robustness\n\
Controls the precision of certain internal processing.\n\
@end table\n\
\n\
Each of @var{accuracy}, @var{persistence}, and @var{robustness} can be set to\n\
a small positive number (typically @leq{}3), to increment the controlled\n\
item; this may increase computation time, but the need to do so should be\n\
rare.\n\
\n\
@end table\n\
\n\
@heading Syntax examples\n\
\n\
Type I low-pass; lower relative error in stop-band; increased @var{accuracy}:\n\
\n\
@example\n\
b=firpm(40, [0 0.4 0.5 1], [1 0], [1 10], 'accuracy', 1);\n\
@end example\n\
\n\
Type II band-pass; amplitude per band-edge; even weight; default\n\
@var{properties}:\n\
\n\
@example\n\
b=firpm(41, [0 0.5 0.6 0.7 0.8 1], [0 0 1 1 0 0]);\n\
@end example\n\
\n\
Type III differentiator:\n\
\n\
@example\n\
b=firpm(40, [0 0.9], [0 0.9], 'class', 'differentiator');\n\
@end example\n\
\n\
Type IV high-pass with gain; @var{class} value abbreviated:\n\
\n\
@example\n\
b=firpm(31, [0 0.5 0.7 1], [0 2.5], 'class', 'antisym');\n\
@end example\n\
\n\
To use other units for @var{f}, divide by the value in those units that\n\
represents the Nyquist-frequency.  For example:\n\
\n\
@example\n\
@group\n\
Fs = 96000;    % Sampling frequency in Hz\n\
Fn = Fs / 2;   % Nyquist frequency in Hz\n\
b = firpm(50, [0 20000 28000 48000] / Fn, [1 0]);\n\
@end group\n\
@end example\n\
\n\
Further syntax examples can be found among the demonstration designs.\n\
\n\
@heading Compatibility\n\
\n\
For compatibility with other implementations, property @var{class} values\n\
`bandpass' and `hilbert' may be used in place of `symmetric' and\n\
`antisymmetric' respectively, and when @var{class} is the last property\n\
given, the property name argument may be ommitted.  For example, the\n\
following are all equivalent:\n\
\n\
@example\n\
@group\n\
b=firpm(30, [0.1 0.9], 1, 'class', 'antisymmetric');\n\
b=firpm(30, [0.1 0.9], 1, 'class', 'hilbert');\n\
b=firpm(30, [0.1 0.9], 1, 'hilbert');\n\
@end group\n\
@end example\n\
\n\
This function is broadly compatible with the Matlab@registeredsymbol{}\n\
function of the same name.  Specific differences exist in the third returned\n\
value, and the mechanism used for controlling filter accuracy (`lgrid' in\n\
Matlab).\n\
\n\
@heading Diagnostics\n\
\n\
If a problem occurs during the computation, a diagnostic message will\n\
normally be displayed.  If this happens, adjusting @var{accuracy},\n\
@var{persistence}, or @var{robustness} may provide the solution.  Some\n\
filters however,  may not be realisable due to machine-precision limitations;\n\
in this case, relaxing the design specification (e.g. reducing the\n\
filter-order) might be an option.\n\
\n\
If the computation fails early on, no value will be returned.  Otherwise,\n\
@var{minimax} is the approximated, minimised, maximum weighted error, and\n\
@var{result} is zero for success, or non-zero if the required accuracy could\n\
not be achieved (in which case @var{minimax} may not be accurate).\n\
\n\
Returned @var{minimax} and @var{result} values could be used in an iterative\n\
algorithm to determine the minimum filter order needed to meet a given\n\
specification.\n\
@ignore\n\
In implementing such an algorithm, giving the @var{target} property can save\n\
a little time by halting the firpm processing early if the target minimax\n\
error can not be achieved.  Giving a @var{target} value also suppresses\n\
diagnostic display, so @var{result} must be used for diagnostics in this\n\
case.\n\
@end ignore\n\
\n\
Note that multiplying @var{w} by a constant affects @var{minimax} similarly.\n\
Hence, in the following example, b1 and b2 are equal, but m1 and m2 are not:\n\
\n\
@example\n\
@group\n\
[b1 m1] = firpm(20, [0 0.4 0.5 1], [1 0], [1 1/8])\n\
[b2 m2] = firpm(20, [0 0.4 0.5 1], [1 0], [8   1])\n\
@end group\n\
@end example\n\
\n\
With this filter design method, transition-band behaviour is not specified,\n\
which can sometimes cause transition-band `anomalies'.  For this reason (and\n\
perhaps others), it is wise to check that the actual response of the computed\n\
filter is satisfactory.  For example:\n\
\n\
@example\n\
@group\n\
b=firpm(40, [0 0.3 0.4 0.6 0.8 1], [0 1 0]);\n\
freqz(b)\n\
@end group\n\
@end example\n\
\n\
For how to prevent transition-band anomalies (and for general discussion of\n\
minimax filter design) see Saram@\"aki.@footnote{ Tapio Saram@\"aki, `Finite\n\
impulse response filter design', Chapter 4 in `Handbook for Digital Signal\n\
Processing', edited by S.  K. Mitra and J. F. Kaiser, John Wiley and Sons,\n\
New York, 1993, pp. 155-277.\n\
(@url{http://www.cs.tut.fi/~ts/Mitra_Kaiser.pdf})}\n\
\n\
@end deftypefn\n\
")
{
  int i, arg(0), nargin(args.length ()), N;

  /* The first 3 parameters are mandatory: */
  if (nargin < 3)
    {
      print_usage ();
      return octave_value_list ();
    }

  /* N: */
  octave_value const & v = args(arg++);
  if (!v.is_real_scalar () || (N = (int)v.scalar_value ()) != v.scalar_value ())
    returnError ("firpm parameter N (filter order) must be an integer");

  /* F[]: */
  ColumnVector freqs;
  if (!args(arg).is_string () && args(arg).is_real_matrix ())
    freqs = args(arg++).vector_value ();
  if (freqs.length () < 2 || (freqs.length () & 1))
    returnError ("firpm parameter F (band-edge frequencies) "
        "must be a real, even-lengthed vector");
  int numBands(freqs.length () >> 1);
  OCTAVE_LOCAL_BUFFER(mmfirBandSpec, bands, numBands);
  for (i = 0; i < numBands; ++i)
    bands[i].freqL = freqs(2*i), bands[i].freqR = freqs(2*i+1);

  /* A[]: */
  ColumnVector amps;
  if (!args(arg).is_string () && (args(arg).is_real_matrix () ||
        (args(arg).is_real_scalar () && numBands == 1)))
    amps = args(arg++).vector_value ();
  bool have2(amps.length () == numBands * 2);
  if (amps.length () != numBands && !have2)
    returnError ("firpm parameter A must be a real vector of "
        "response amplitudes: one per band, or per band-edge");
  for (i = 0; i < numBands; ++i)
    if (have2)
      bands[i].ampL = amps(2*i), bands[i].ampR = amps(2*i+1);
    else bands[i].ampL = bands[i].ampR = amps(i);

  /* W[]: */
  if (arg < nargin && !args(arg).is_string () && (
        args(arg).is_real_matrix () ||
        (args(arg).is_real_scalar () && numBands == 1)))
    {
      ColumnVector weights(args(arg++).vector_value ());
      bool have2(weights.length () == numBands * 2);
      if (weights.length () != numBands && !have2)
        returnError ("firpm parameter W must be a real vector of "
            "error weights: one per band, or per band-edge");
      for (i = 0; i < numBands; ++i)
        if (have2)
          bands[i].weightL = weights(2*i), bands[i].weightR = weights(2*i+1);
        else bands[i].weightL = bands[i].weightR = weights(i);
    }
  else for (i = 0; i < numBands; ++i)
    bands[i].weightL = bands[i].weightR = 1;

  /* PROPERTIES: */
  static char const * const propertyNames[] =
    { "ACCURACY", "PERSISTENCE", "ROBUSTNESS", "TARGET", "FLAGS", "CLASS" };
  double realProperties[] = { 0, 0, 0, 0, 0 };
  int Class(0);
  while (arg < nargin && args(arg).is_string ())
    {
      std::string name(propertyNames[AL(propertyNames)-1]);
      if (arg + 1 < nargin)
        name = args(arg++).string_value ();
      std::transform(name.begin(), name.end(), name.begin(), ::toupper);
      for (i = 0; i < AL(propertyNames) && name != propertyNames[i]; ++i);
      if (!(i < AL(propertyNames)))
        returnError ("firpm unknown PROPERTIES name `%s'", name.c_str ());
      if (i < AL(realProperties))
        {
          if (!args(arg).is_real_scalar ())
            returnError ("firpm property %s must be a real scalar",  name.c_str ());
          realProperties[i] = args(arg++).double_value ();
        }
      else
        {
          char const * names[] =
            {
              "symmetric", "bandpass",
              "antisymmetric", "hilbert",
              "differentiator"
            };
          std::string s(args(arg).is_string () ? args(arg++).string_value () : ".");
          for (;Class < AL(names) &&
              std::string(names[Class]).substr (0, s.size()) != s; ++Class);
          if (!(Class < AL(names)))
            returnError ("firpm property CLASS must be in "
                "{'%s' (or '%s'), '%s' (or '%s'), '%s'}",
                names[0], names[1], names[2], names[3], names[4]);
          Class >>= 1; /* Shift away alternative names. */
        }
    }

  /* If any args remain at this point, then something has gone awry: */
  if (arg < nargin)
    {
      std::ostringstream os;
      args(arg).print (os, true);
      std::string s(os.str ());
      size_t p = s.find_last_not_of (" \t\r\n");
      if (p != std::string::npos)
        s.erase (p+1);
      returnError ("firpm argument # %i is invalid: %s", arg+1, s.c_str ());
    }

  /* Convert N from filter-order to # of taps; attempt to generate the filter:*/
  OCTAVE_LOCAL_BUFFER (double, h, ++N);
  mmfirReport report(mmfir (h, (mmfirFilterClass)Class, N, numBands, bands,
        realProperties[0], realProperties[1], realProperties[2],
        realProperties[3], NINT (realProperties[4])));

  /* Warn if any controls were clamped: */
  for (i = 0; i < AL(report.controls); ++i)
    if (report.controls[i] != realProperties[i])
      warning ("firpm property %s was clamped to %g",
          propertyNames[i], report.controls[i]);

  /* Check if a filter was generated: */
  if (report.result >= mmfirInvocationError)
    returnError ("firpm invocation error: %s", report.text);
  else if (report.result >= mmfirError)
    returnError ("firpm failed to make a filter (e=%i)", report.result);

  /* Missed-target is handled silently: */
  if (report.result == mmfirMissedTarget)
    N = 0; /* Return empty B. */
  else /* Check if a warning should be issued: */
    if (report.result >= mmfirWarning)
      warning ("firpm %s (e=%i)", report.text, report.result);

  /* Copy filter coefficients to an Octave column-vector: */
  ColumnVector b(N);
  while (N--) b(N) = h[N];

  /* Return [B, MINIMAX, RESULT]: */
  octave_value_list retval;
  retval(0) = octave_value(b);
  retval(1) = octave_value(report.minimax);
  retval(2) = octave_value(report.result); /* Numeric (not textual) result. */
  return retval;
}



/* Test and demo scripts for GNU Octave:

%!error <firpm parameter N \(filter order\) must be an integer> firpm (1.1,2,3);
%!error <firpm parameter N \(filter order\) must be an integer> firpm (i,2,3);
%!error <firpm parameter N \(filter order\) must be an integer> firpm ([1 2],2,3);

%!error <firpm parameter F \(band-edge frequencies\) must be a real, even-lengthed vector> firpm (1,2,3);
%!error <firpm parameter F \(band-edge frequencies\) must be a real, even-lengthed vector> firpm (2,[1 2 3],3);
%!error <firpm parameter F \(band-edge frequencies\) must be a real, even-lengthed vector> firpm (2,[1 i],3);

%!error <firpm parameter A must be a real vector of response amplitudes: one per band, or per band-edge> firpm (1,[1 2],i);
%!error <firpm parameter A must be a real vector of response amplitudes: one per band, or per band-edge> firpm (1,[1 2],[]);
%!error <firpm parameter A must be a real vector of response amplitudes: one per band, or per band-edge> firpm (1,[1 2],[1 2 3]);
%!error <firpm parameter A must be a real vector of response amplitudes: one per band, or per band-edge> firpm (1,[1 2],[1 i]);

%!error <firpm argument # 4 is invalid: \(0,1\)> firpm (1,[1 2],1,i);
%!error <firpm parameter W must be a real vector of error weights: one per band, or per band-edge> firpm (1,[1 2],1,[]);
%!error <firpm parameter W must be a real vector of error weights: one per band, or per band-edge> firpm (1,[1 2],1,[1 2 3]);

%!error <firpm property CLASS must be in {'symmetric' \(or 'bandpass'\), 'antisymmetric' \(or 'hilbert'\), 'differentiator'}> firpm (1,[1 2],1,[1 2],'diffi');
%!error <firpm property CLASS must be in {'symmetric' \(or 'bandpass'\), 'antisymmetric' \(or 'hilbert'\), 'differentiator'}> firpm (1,[1 2],1,[1 2],'hilberts');

%!error <firpm property ACCURACY must be a real scalar> firpm (1,[1 2],1,[1 2],'accuracy', 'antisymmetric');
%!error <firpm property PERSISTENCE must be a real scalar> firpm (1,[1 2],1,[1 2],'persistence', [1 1]);

%!warning <firpm property ROBUSTNESS was clamped to 0> firpm (1,[.1 .9],1,[1 2],'robustness',-1);
%!error <firpm failed to make a filter \(e=4\)> firpm (2,[.1 .9],1);
%!error <firpm argument # 4 is invalid: {}\(0x0\)> firpm (2,[0 1],[1 0],{},1);

%!error <band-edge frequencies must increase in \[0,1\]> firpm (1,[1 2],1,[1 2],'diff');

%!test b0=[
%!   0.01402447013732010
%!  -0.00187656470348210
%!  -0.03037226560233987
%!   0.01238671724689763
%!   0.01701736750357643
%!   0.00130000669025103
%!   0.01887058201233184
%!  -0.04786566643294526
%!  -0.02646470269612746
%!   0.05579339917757684
%!   0.00111880676596655
%!   0.05887956174380488
%!  -0.03589764058661398
%!  -0.22983989436822705
%!   0.17175369560988285
%!   0.28425948627067166 ]';
%! [b m r]=firpm(31 ,[0 .3 .4 .7 .8 1], [0 1 0], [10 1 10], 'accuracy', 1, 'class', 'a');
%! assert(b',[-b0 fliplr(b0)],5e-9);
%! assert(r,0,0); assert(m/0.059734,1,5e-5);

%!test [b m r]=firpm(40, [0 .5 .6 1], [1 0]);
%! assert(r,0,0); assert(m/0.010304,1,5e-5);

%!test [b m r]=firpm(40, [0 .25 .3 .6 .65 1], [0 1 0]);
%! assert(r,0,0); assert(m/0.055834,1,5e-5);

%!test [b m r]=firpm(30, [0 .2 .4 .6 .8 1], [1 .5 0]);
%! assert(r,0,0); assert(m/5.6274e-04,1,5e-5);

%!test
%! [b m r]=firpm(40, [0 .2 .3 .4 .5 .6 .7 .8 .9 1],[1 0 1 0 1],[11 100 7 10 5]);
%! assert(r,0,0); assert(m/0.25720,1,5e-5);

%!test [b m r]=firpm(80, [0 .2 .3 1], [1 -1]);
%! assert(r,0,0); assert(m/6.6123e-04,1,5e-5);

%!test [b m r]=firpm(40, [0 2.5/pi], [0 2.5], [.5], 'class', 'differentiator');
%! assert(r,0,0); assert(m/2.5226e-06,1,5e-5);

%!test [b m r]=firpm(59, [0 .2 .3 1], [0 1 0 0], 'd');
%! assert(r,0,0); assert(m/0.0073785,1,5e-5);

%!test [b m r]=firpm(31, [0 .5 .7 1], [0 1], 'class', 'antisymmetric');
%! assert(r,0,0); assert(m/0.0015660,1,5e-5);

%!test assert(firpm(31, [0 .5 .7 1], [0 1], 'class', 'antisymmetric'), firpm(31, [0 .5 .7 1], [0 1], 'hilbert'))

%!test [b m r]=firpm(30, [.1 .9], 1, 'class', 'antisymmetric');
%! assert(r,0,0); assert(m/0.0027064,1,5e-5);

%!test [b m r]=firpm(1000, [0 .4 .41 1], [1 0]);
%! assert(r,0,0); assert(m/5.2892e-05,1,5e-5);

%!test [b m r]=firpm(120, [0 .5 .55 .75 .75 1], [1 0 0], [1 1 4 64 64 64]);
%! assert(r,0,0); assert(m/0.0048543,1,5e-5);

%!test [b m r]=firpm(11, [0 2*.45], .5, 'class', 'symmetric');
%! assert(r,0,0); assert(m/0.045066,1,5e-5);

%!test assert(firpm(11, [0 2*.45], .5, 'class', 'symmetric'), firpm(11, [0 2*.45], .5, 'bandpass'))

%!demo
%!
%! b=firpm(40, [0 .5 .6 1], [1 0]);
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 1 -50 2])
%! title('firpm low-pass FIR filter design');
%! ylabel('Gain (dB)'); xlabel('Frequency (normalised)')
%! axes('position',[.21 .55 .3 .2])
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 .52 -(e=1.1e-1) e])
%! title('Pass-band detail')
%! axes('position',[.21 .2 .3 .2])
%! plot(b); grid('on')
%! title('Impulse response')
%! axis([1 length(b) -.2 .6])
%! %--------------------------------------------------
%! % Figure shows transfer and impulse-response of
%! % low-pass filter design.

%!demo
%!
%! b=firpm(40, [0 .25 .3 .6 .65 1], [0 1 0]);
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! subplot(2,2,1:2)
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 1 -50 5])
%! title('firpm band-pass FIR filter design');
%! ylabel('Gain (dB)'); xlabel('Frequency (normalised)')
%! subplot(2,2,3)
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([.29 .61 -(e=6e-1) e])
%! title('Pass-band detail')
%! subplot(2,2,4)
%! plot(b); grid('on')
%! title('Impulse response')
%! axis([1 length(b) -.3 .4])
%! %--------------------------------------------------
%! % Figure shows transfer and impulse-response of band-pass
%! % filter design.

%!demo
%!
%! b=firpm(30, [0 .2 .4 .6 .8 1], [1 .5 0]);
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! plot(f/pi, abs(h)); grid('on'); axis([0 1 -.1 1.1])
%! title('firpm multi-level FIR filter design');
%! ylabel('Magnitude response'); xlabel('Frequency (normalised)')
%! axes('position',[.21 .2 .3 .2])
%! plot(f/pi, abs(h)); grid('on'); axis([.39 .61 .5-(e=1e-3) .5+e])
%! title('Mid-band detail')
%! axes('position',[.55 .65 .3 .2])
%! plot(f/pi, abs(h)); grid('on'); axis([0 .21 1-(e=1e-3) 1+e])
%! title('Pass-band detail')
%! %--------------------------------------------------
%! % Figure shows transfer of multi-level filter design.

%!demo
%!
%! b=firpm(40, [0 .2 .3 .4 .5 .6 .7 .8 .9 1], [1 0 1 0 1], [11 100 7 10 5]);
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! subplot(2,1,1)
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 1 -60 5]); set(gca, 'xtick', [0:.1:1])
%! title('firpm multi-band, multi-weight FIR filter design');
%! ylabel('Gain (dB)'); xlabel('Frequency (normalised)')
%! subplot(2,1,2)
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 1 -.5 .5]); set(gca, 'xtick', [0:.1:1])
%! title('Pass-band detail')
%! %--------------------------------------------------
%! % Figure shows transfer of multi-band, multi-weight filter design.

%!demo
%!
%! b=firpm(80, [0 .2 .3 1], [1 -1]);
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 1 -60 5])
%! title('firpm notch FIR filter design');
%! ylabel('Gain (dB)'); xlabel('Frequency (normalised)')
%! axes('position',[.5 .55 .3 .2])
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 1 -(e=8e-3) e])
%! title('Pass-band detail')
%! axes('position',[.5 .2 .3 .2])
%! plot(b); grid('on')
%! title('Impulse response')
%! axis([1 length(b) -.55 .5])
%! %--------------------------------------------------
%! % Figure shows transfer and impulse-response of
%! % notch filter design.

%!demo
%!
%! b=firpm(40, [0 2.5/pi], [0 2.5], 'class', 'differentiator');
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! subplot(2,1,1)
%! plot(f, abs(h)); grid('on')
%! title('firpm differentiator FIR filter design');
%! ylabel('Magnitude response'); xlabel('Frequency (radians/sample)')
%! axis([0 pi 0 pi])
%! subplot(2,1,2)
%! plot(f, abs(abs(h)./f-1)); grid('on')
%! axis([0 2.5 0 1e-6])
%! title('Pass-band error (inverse-f weighted)')
%! %--------------------------------------------------
%! % Figure shows transfer of differentiator filter design.
%! % above: full-band
%! % below: detail of pass-band error (inverse-f weighted)

%!demo
%!
%! b=firpm(31, [0 .5 .7 1], [0 1], 'class', 'antisymmetric');
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 1 -80 5])
%! title('firpm odd-order high-pass FIR filter design');
%! ylabel('Gain (dB)'); xlabel('Frequency (normalised)')
%! axes('position',[.2 .65 .3 .2])
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([.69 1 -(e=2e-2) e])
%! title('Pass-band detail')
%! axes('position',[.57 .3 .3 .2])
%! plot(b); grid('on')
%! title('Impulse response')
%! axis([1 length(b) -.4 .4])
%! %--------------------------------------------------
%! % Figure shows transfer and impulse-response of
%! % odd-order high-pass filter design.

%!demo
%!
%! b=firpm(30, [.1 .9], 1, 'class', 'antisymmetric');
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! plot(f/pi, abs(h)); grid('on')
%! title('firpm hilbert transformer FIR filter design');
%! ylabel('Magnitude response'); xlabel('Frequency (normalised)')
%! axis([0 1 0 1.1])
%! axes('position',[.3 .4 .45 .2])
%! plot(f/pi, abs(h)); grid('on')
%! axis([0 1 1-(e=4e-3) 1+e])
%! title('Pass-band detail')
%! %--------------------------------------------------
%! % Figure shows transfer of hilbert filter design.

%!demo
%!
%! b=firpm(1000, [0 .4 .41 1], [1 0]);
%!
%! clf
%! [h,f]=freqz(b,1,2^17);
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! title('firpm brick-wall low-pass FIR filter design');
%! ylabel('Gain (dB)'); xlabel('Frequency (normalised)')
%! axis([0 1 -100 5])
%! axes('position',[.55 .6 .3 .2])
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! title('Details')
%! axis([.39 .401 -(e=1e-3) e])
%! axes('position',[.55 .3 .3 .2])
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([.409 .43 -86 -85])
%! %--------------------------------------------------
%! % Figure shows transfer of brick-wall low-pass filter design.

%!demo
%!
%! b=firpm(120, [0 .5 .55 .75 .75 1], [1 0 0], [1 1 4 64 64 64]);
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 1 -120 5])
%! title('firpm low-pass FIR filter design with shaped stop-band error');
%! ylabel('Gain (dB)'); xlabel('Frequency (normalised)')
%! axes('position',[.21 .55 .3 .2])
%! plot(f/pi, 20*log10(abs(h))); grid('on')
%! axis([0 .52 -(e=7e-2) e])
%! title('Pass-band detail')
%! %--------------------------------------------------
%! % Figure shows transfer of low-pass filter design
%! % with shaped error-weight in the stop-band.

%!demo
%!
%! b1=firpm(11, [0 2*.45], .5);
%! b=zeros(length(b1)*2-1,1); b(1:2:length(b))=b1; b(length(b1))=.5;
%!
%! clf
%! [h,f]=freqz(b,1,2^14);
%! plot(f/pi, abs(h)); grid('on')
%! axis([0 1 0 1.1])
%! title('firpm half-band FIR filter design');
%! ylabel('Magnitude'); xlabel('Frequency (normalised)')
%! set(gca, 'xtick', [0:.1:1]);
%! set(gca, 'ytick', [0:.1:1]);
%! axes('position',[.58 .35 .3 .5])
%! plot(b, '-o'); grid('on')
%! title('Impulse response')
%! axis([1 length(b) -.15 .55])
%! %--------------------------------------------------
%! % Figure shows transfer and impulse-response of
%! % half-band filter design using an approach that
%! % uses less CPU and yields exact zero coefficients.

*/

#endif
